unit CompareObjsImpl;

interface

uses AQtest_TLB,CompareObjs_TLB, ComObj, ActiveX, Windows, Classes, XpDom, XpBase, XpParser;

type
  TCompareObjs = class(TAutoIntfObject, ICompareObjs)
  private
    FStrings:TStrings;
    XpObjModel1: TXpObjModel;
    XpObjModel2: TXpObjModel;
    FFullName1:String;
    FFullName2:String;
  protected
    function FullName(Node:TxpNode):String;
    procedure CheckNodes(const Node1,Node2:TXpElement);
    procedure  CheckProperties(const PropName: WideString);
  public
    destructor Destroy; override;
    constructor Create;
    function  Compare(const AObject: IDispatch; const Name: WideString; const PropName: WideString): WordBool; safecall;
  end;

var
  CompareObjsPlugIn: TCompareObjs;
  PIManager: IPIManager;

implementation

uses sysutils;

resourcestring
  SComparingObjectsTo = 'Comparing Objects %s to %s';
  SDoesNotHaveMatchingPropertyField = '%s Does Not Have Matching Property/Field: (%s)';
  SDoesNotMatch = '%s Does Not Match: (%s,%s)';
  SIsNotTheSameObjectAs = '%s is not the same object as %s';
  SPropertyWasNotFound = 'Property (%s) was not found';
  SStoredObject = 'Stored Object (%s)';

function ExtractTokens(const S : String;
                        TokenList     : TStrings) : Cardinal;
const
  Delims = ' ';
var
  P1,
  P2     : Cardinal;
  AStr,
  TmpStr : ShortString;
begin
  if (S = '') then begin
    Result := 0;
    Exit;
  end;

  TokenList.Clear;

  TmpStr := Trim(S);

  P1 := 1;
  repeat
    while (Pos(TmpStr[P1], Delims) = 0) and
          (P1 <= Length(TmpStr)) do
      Inc(P1);
    if (P1 < Length(TmpStr)) then
    begin
        AStr := Copy(TmpStr, 1, P1-1);
        TokenList.Add(AStr);
        System.Delete(TmpStr, 1, P1);
        P1 := 1;
    end
    else
    begin
      P2 := Length(TmpStr);
      if (Pos(TmpStr[P2], Delims) = 0) then
        TokenList.Add(Copy(TmpStr, 1, P2))
      else
        TokenList.Add(Copy(TmpStr, 1, P2-1));
      TmpStr := '';
    end;
  until (Length(TmpStr) = 0);
  Result := TokenList.Count;
end;

{ TCompareObjs }

procedure TCompareObjs.CheckNodes(const Node1, Node2: TXpElement);
var
  I: Integer;
  XpList1,
  XpList2:TXpNodeList;
begin

  if Node1.HasAttribute('VarType') then
  begin
    XPList1 :=  Node1.GetNodesByNodeType(CData_Section_Node);
    if XPList1.Length > 0 then
    begin
      if Node2 = nil then
      begin
           FStrings.Add(Format(SDoesNotHaveMatchingPropertyField,[
           FullName(Node1),
           TXpCDataSection(XpList1.Item(0)).Data]));
      end
      else
      begin
        XPList2 := Node2.GetNodesByNodeType(CData_Section_Node);
        if XPList2.Length > 0 then
        begin
          if AnsiCompareStr(TXpCDataSection(XpList1.Item(0)).Data,
             TXpCDataSection(XpList2.Item(0)).Data) <> 0 then
          begin
            FStrings.Add(Format(SDoesNotMatch,[
             Node1.LocalName,
             TXpCDataSection(XpList1.Item(0)).Data,
             TXpCDataSection(XpList2.Item(0)).Data]));
          end;
        end
        else
        begin
          FStrings.Add(Format(SDoesNotHaveMatchingPropertyField,[
           FullName(Node1),
           TXpCDataSection(XpList1.Item(0)).Data]));
        end;
      end;
    end;
  end
  else
  begin
    XpList1 := Node1.ChildNodes;
    for I := 0 to Xplist1.Length - 1 do    // Iterate
    begin
      CheckNodes(TXpElement(XpList1.Item(i)),TXpElement(Node2).FindElement(XpList1.Item(i).NodeName));
    end;    // for
  end;
end;

procedure TCompareObjs.CheckProperties(const PropName: WideString);
var
  I: Integer;
  Props:TStrings;
  XpList:TXpNodeList;
  pValue1,
  pValue2:String;
begin
  Props := TStringList.Create;
  ExtractTokens(PropName,Props);
  for I := 0 to Props.Count - 1 do    // Iterate
  begin
    XpList := XpObjModel1.DocumentElement.SelectNodes('//'+Props[i]).Item(0).GetNodesByNodeType(CData_Section_Node);
    if XpList.Length = 0 then
    begin
      PIManager.Log.Error(Format(SPropertyWasNotFound,[Props[i]]),'',2,0,$80000008,$80000005);
      continue;
    end
    else
      pValue1 := TXpCDataSection(XpList.Item(0)).Data;

    XpList := XpObjModel2.DocumentElement.SelectNodes('//'+Props[i]).Item(0).GetNodesByNodeType(CData_Section_Node);
    if XpList.Length = 0 then
    begin
      PIManager.Log.Error(Format(SPropertyWasNotFound,[Props[i]]),'',2,0,$80000008,$80000005);
      continue;
    end
    else
      pValue2 := TXpCDataSection(XpList.Item(0)).Data;
    if AnsiCompareStr(pValue1,pValue2) <> 0 then
    begin
      FStrings.Add(Format(SDoesNotMatch,[
       Props[i],
       pValue1,
       pValue2]));
    end;

  end;    // for
end;

function  TCompareObjs.Compare(const AObject: IDispatch; const Name: WideString;
  const PropName: WideString): WordBool;
var
  XpList:TXpNodeList;
  dir:String;
begin
  PIManager.Objects.Save(AObject,'AQACmpr1','');
  Dir := PIManager.Project.Directory;
  XpObjModel1.LoadDataSource(Dir + 'AQACmpr1.XML');
  XpObjModel2.LoadDataSource(Dir + Name +'.XML');
  XpList := XpObjModel1.DocumentElement.SelectNodes('//FullName').Item(0).GetNodesByNodeType(CData_Section_Node);
  if XpList.Length > 0 then
    FFullName1 := TXpCDataSection(XpList.Item(0)).Data
  else
    FFullName1 := (AObject as IAQTestObject).FullName;
  XpList := XpObjModel2.DocumentElement.SelectNodes('//FullName').Item(0).GetNodesByNodeType(CData_Section_Node);
  if XpList.Length > 0 then
    FFullName2 := TXpCDataSection(XpList.Item(0)).Data
  else
    FFullName2 := Format(SStoredObject,[Name]);
  FStrings.Clear;
  FStrings.Add(Format(SComparingObjectsTo,[FFullName1,FFullName2]));
  FStrings.Add('');
  if Trim(PropName) = '' then
    CheckNodes(XpObjModel1.DocumentElement,XpObjModel2.DocumentElement)
  else
    CheckProperties(PropName);
  Result := not (FStrings.Count > 2);
  if not Result then
     PIManager.Log.Warning(Format(SIsNotTheSameObjectAs,[FFullName1, FFullName2]),FStrings.Text,2,0,$80000008,$80000005);
end;

function GetModuleTypeLib: ITypeLib;

  function GetModuleFileName: string;
  var
    Buffer: array[0..MAX_PATH] of Char;
  begin
    SetString(Result, Buffer, Windows.GetModuleFileName(HInstance,
      Buffer, SizeOf(Buffer)));
  end;

  function LoadTypeLibrary(const ModuleName: string): ITypeLib;
  begin
    OleCheck(LoadTypeLib(PWideChar(WideString(ModuleName)), Result));
  end;

begin
  Result := LoadTypeLibrary(GetModuleFileName);
end;


constructor TCompareObjs.Create;
begin
  inherited Create(GetModuleTypeLib,ICompareObjs);
  FStrings := TStringList.Create;
  XpObjModel1 := TXpObjModel.Create(nil);
  XpObjModel2 := TXpObjModel.Create(nil);
end;

destructor TCompareObjs.Destroy;
begin
  XpObjModel2.Free;
  XpObjModel1.Free;

  FStrings.Free;
  inherited;
end;

function TCompareObjs.FullName(Node: TxpNode): String;
begin
  if Node.ParentNode.ParentNode = nil then
  begin
    Result := Node.LocalName;
  end
  else
  begin
    Result := Format('%s:%s',[FullName(Node.ParentNode),Node.LocalName]);
  end;
end;

end.
